@@ -65,7 +65,7 @@ struct MixLoopState
 		// If there is no interpolation happening, there is no lookahead happening the sample read-out is exact.
 		if(chn.dwFlags[CHN_LOOP] && chn.resamplingMode != SRCMODE_NEAREST)
 		{
-			const bool inSustainLoop = chn.InSustainLoop();
+			const bool inSustainLoop = chn.InSustainLoop() && chn.nLoopStart == chn.pModSample->nSustainStart && chn.nLoopEnd == chn.pModSample->nSustainEnd;
 
 			// Do not enable wraparound magic if we're previewing a custom loop!
 			if(inSustainLoop || chn.nLoopEnd == chn.pModSample->nLoopEnd)
@@ -89,7 +89,7 @@ struct MixLoopState
 	// Check how many samples can be rendered without encountering loop or sample end, and also update loop position / direction
 	MPT_FORCEINLINE uint32 GetSampleCount(ModChannel &chn, uint32 nSamples, bool ITPingPongMode) const
 	{
-		int32 nLoopStart = chn.dwFlags[CHN_LOOP] ? chn.nLoopStart : 0;
+		const int32 nLoopStart = chn.dwFlags[CHN_LOOP] ? chn.nLoopStart : 0;
 		SamplePosition nInc = chn.increment;
 
 		if(nSamples <= 0 || nInc.IsZero() || !chn.nLength || !samplePointer)
@@ -199,35 +199,24 @@ struct MixLoopState
 		{
 			if(nPos.GetUInt() >= lookaheadStart)
 			{
-#if 0
-				const uint32 oldCount = nSmpCount;
-
-				// When going backwards - we can only go back up to lookaheadStart.
-				// When going forwards - read through the whole pre-computed wrap-around buffer if possible.
-				// TODO: ProTracker sample swapping needs hard cut at sample end.
-				int32 samplesToRead = nInc.IsNegative()
-					? (nPosInt - lookaheadStart)
-					//: 2 * InterpolationMaxLookahead - (nPosInt - mixLoopState.lookaheadStart);
-					: (chn.nLoopEnd - nPosInt);
-				//LimitMax(samplesToRead, chn.nLoopEnd - chn.nLoopStart);
-				nSmpCount = SamplesToBufferLength(samplesToRead, chn);
-				Limit(nSmpCount, 1u, oldCount);
-#else
-				if (nInc.IsNegative())
+				if(nInc.IsNegative())
 				{
 					nSmpCount = DistanceToBufferLength(SamplePosition(lookaheadStart, 0), nPos, nInv);
-				} else
+					chn.pCurrentSample = lookaheadPointer;
+				} else if(nPosInt <= chn.nLoopEnd)
 				{
 					nSmpCount = DistanceToBufferLength(nPos, SamplePosition(chn.nLoopEnd, 0), nInv);
+					chn.pCurrentSample = lookaheadPointer;
+				} else
+				{
+					nSmpCount = DistanceToBufferLength(nPos, SamplePosition(chn.nLength, 0), nInv);
 				}
-#endif
-				chn.pCurrentSample = lookaheadPointer;
 				checkDest = false;
 			} else if(chn.dwFlags[CHN_WRAPPED_LOOP] && isAtLoopStart)
 			{
 				// We just restarted the loop, so interpolate correctly after wrapping around
 				nSmpCount = DistanceToBufferLength(nPos, SamplePosition(nLoopStart + InterpolationMaxLookahead, 0), nInv);
-				chn.pCurrentSample = lookaheadPointer + (chn.nLoopEnd - nLoopStart) * chn.pModSample->GetBytesPerSample();
+				chn.pCurrentSample = lookaheadPointer + (chn.nLength - nLoopStart) * chn.pModSample->GetBytesPerSample();
 				checkDest = false;
 			} else if(nInc.IsPositive() && static_cast<SmpLength>(nPosDest) >= lookaheadStart && nSmpCount > 1)
 			{
@@ -255,9 +244,9 @@ struct MixLoopState
 			}
 		}
 
-		Limit(nSmpCount, 1u, nSamples);
+		Limit(nSmpCount, uint32(1u), nSamples);
 
-#ifdef _DEBUG
+#ifdef MPT_BUILD_DEBUG
 		{
 			SmpLength posDest = (nPos + nInc * (nSmpCount - 1)).GetUInt();
 			MPT_MAYBE_CONSTANT_IF(posDest < 0 || posDest > chn.nLength)
@@ -376,26 +365,40 @@ void CSoundFile::CreateStereoMix(int count)
 				chn.nROfs = chn.nLOfs = 0;
 				pbuffer += nSmpCount * 2;
 				naddmix = 0;
-			} else
+			}
+#ifdef MODPLUG_TRACKER
+			else if(m_SamplePlayLengths != nullptr)
+			{
+				// Detecting the longest play time for each sample for optimization
+				chn.position += chn.increment * nSmpCount;
+				size_t smp = std::distance<const ModSample *>(Samples, chn.pModSample);
+				if(smp < m_SamplePlayLengths->size())
+				{
+					m_SamplePlayLengths->at(smp) = std::max(m_SamplePlayLengths->at(smp), chn.position.GetUInt());
+				}
+			}
+#endif
+			else
 			{
 				// Do mixing
 				mixsample_t *pbufmax = pbuffer + (nSmpCount * 2);
-				chn.nROfs = - *(pbufmax-2);
-				chn.nLOfs = - *(pbufmax-1);
+				chn.nROfs = -*(pbufmax - 2);
+				chn.nLOfs = -*(pbufmax - 1);
 
-#ifdef _DEBUG
+#ifdef MPT_BUILD_DEBUG
 				SamplePosition targetpos = chn.position + chn.increment * nSmpCount;
 #endif
 				MixFuncTable::Functions[functionNdx | (chn.nRampLength ? MixFuncTable::ndxRamp : 0)](chn, m_Resampler, pbuffer, nSmpCount);
-#ifdef _DEBUG
+#ifdef MPT_BUILD_DEBUG
 				MPT_ASSERT(chn.position.GetUInt() == targetpos.GetUInt());
 #endif
 
-				chn.nROfs += *(pbufmax-2);
-				chn.nLOfs += *(pbufmax-1);
+				chn.nROfs += *(pbufmax - 2);
+				chn.nLOfs += *(pbufmax - 1);
 				pbuffer = pbufmax;
 				naddmix = 1;
 			}
+
 			nsamples -= nSmpCount;
 			if (chn.nRampLength)
 			{
@@ -425,7 +428,7 @@ void CSoundFile::CreateStereoMix(int count)
 					// Test case: PTInstrSwap.mod
 					const ModSample &smp = Samples[chn.nNewIns];
 					chn.pModSample = &smp;
-					chn.pCurrentSample = smp.pSample;
+					chn.pCurrentSample = smp.samplev();
 					chn.dwFlags = (chn.dwFlags & CHN_CHANNELFLAGS) | smp.uFlags;
 					chn.nLength = smp.uFlags[CHN_LOOP] ? smp.nLoopEnd : smp.nLength;
 					chn.nLoopStart = smp.nLoopStart;
@@ -546,7 +549,7 @@ void CSoundFile::ProcessPlugins(uint32 nCount)
 			if(!plugin.IsMasterEffect() && !plugin.pMixPlugin->ShouldProcessSilence() && !(plugin.pMixPlugin->m_MixState.dwFlags & SNDMIXPLUGINSTATE::psfHasInput))
 			{
 				// If plugin has no inputs and isn't a master plugin, we shouldn't let it process silence if possible.
-				// I have yet to encounter a plugin which actually sets this flag.
+				// I have yet to encounter a VST plugin which actually sets this flag.
 				bool hasInput = false;
 				for(PLUGINDEX inPlug = 0; inPlug < plug; inPlug++)
 				{
